package fachada;

import java.net.MalformedURLException;

import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.util.Calendar;

import dados.exception.AvaliacaoInvalidaException;
import dados.exception.EmailInvalidoException;
import dados.exception.FilialInvalidaException;
import dados.exception.LocacaoInvalidaException;
import dados.exception.PeriodoInvalidoException;
import dados.exception.PlanoInvalidoException;
import dados.exception.ProdutoInvalidoException;
import dados.exception.PromocaoInvalidaException;
import dados.exception.ReservaInvalidaException;
import dados.filial.Endereco;
import dados.filial.Filial;
import dados.locacao.Copia;
import dados.locacao.Locacao;
import dados.locacao.Periodo;
import dados.locacao.Promocao;
import dados.locacao.Reserva;
import dados.produto.AvaliacaoProduto;
import dados.produto.ClassificacaoPreco;
import dados.produto.CriterioBusca;
import dados.produto.LinkFoto;
import dados.produto.Produto;
import dados.usuario.Cliente;
import dados.usuario.Plano;
import dados.usuario.Telefone;
import dados.usuario.Usuario;

import servicos.InterfaceGeral;
import servicos.InterfaceWebBD;
import util.IteratorBidirecional;

import negocio.exception.AvaliacaoExistenteException;
import negocio.exception.AvaliacaoInexistenteException;
import negocio.exception.ClassificacaoExistenteException;
import negocio.exception.ClassificacaoInexistenteException;
import negocio.exception.CopiaExistenteException;
import negocio.exception.CopiaInexistenteException;
import negocio.exception.CopiaLocadaNoPeriodoException;
import negocio.exception.CopiaReservadaNoPeriodoException;
import negocio.exception.FilialExistenteException;
import negocio.exception.FilialInexistenteException;
import negocio.exception.FilialNaoLogadaException;
import negocio.exception.LinkFotoExistenteException;
import negocio.exception.LocacaoExistenteException;
import negocio.exception.PlanoExistenteException;
import negocio.exception.PlanoInexistenteException;
import negocio.exception.ProdutoExistenteException;
import negocio.exception.ProdutoInexistenteException;
import negocio.exception.PromocaoExistenteException;
import negocio.exception.PromocaoInexistenteException;
import negocio.exception.ReservaExistenteException;
import negocio.exception.ReservaInexistenteException;
import negocio.exception.SenhaInvalidaException;
import negocio.exception.UsuarioExistenteException;
import negocio.exception.UsuarioInexistenteException;
import negocio.exception.UsuarioLogadoException;
import negocio.exception.UsuarioNaoLogadoException;
import negocio.exception.UsuarioSemPermissaoException;

import exception.ServidorIndisponivelException;

public class Fachada {

	private static Fachada fachada;
	private static String chave = "ABQIAAAAUgF7fGBiu_MVgbFod-1KvRRoLeG5xBMISP8cweoMcB1utWxp3RQ0VtbLHPBIELhCBDnnKreNX4BhQw";

	private InterfaceWebBD servico;
	
	private static final double taxaEntrega = 4.0;

	public Fachada() throws ServidorIndisponivelException {
		try {
			if(System.getSecurityManager() != null){
				System.setSecurityManager(new RMISecurityManager());
				System.out.println("RMI Security Manager ON");
			}
			
			String ipConfig = String.format("rmi://%s:1099/",InterfaceGeral.ip);
			this.servico = (InterfaceWebBD) Naming.lookup(ipConfig + "InterfaceWebBD");
		} catch (MalformedURLException e) {
			throw new ServidorIndisponivelException();
		} catch (RemoteException e) {
			e.printStackTrace();
			throw new ServidorIndisponivelException();
		} catch (NotBoundException e) {
			throw new ServidorIndisponivelException();
		}
	}
	
	public static String getChave(){
		return Fachada.chave;
	}
	
	public static double getTaxaEntrega(){
		return Fachada.taxaEntrega;
	}

	public static Fachada getFachada() throws ServidorIndisponivelException{
		if(Fachada.fachada == null){
			Fachada.fachada = new Fachada();
		}

		return Fachada.fachada;
	}

	public long logarUsuario(long usuario, String senha) throws NumberFormatException, RemoteException, UsuarioLogadoException, UsuarioInexistenteException, SenhaInvalidaException{
		if(senha != null){
			try{
				return this.servico.logarUsuario(usuario, senha);
			}catch(RemoteException e){
				Fachada.fachada = null;
				throw e;
			}
		}else{
			throw new IllegalArgumentException();
		}
	}

	public Usuario getUsuario(long sessao, long cpf) throws RemoteException, UsuarioNaoLogadoException, UsuarioInexistenteException, UsuarioSemPermissaoException{
		try{
			return this.servico.buscarUsuario(sessao, cpf);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void logoff(Long sessao) throws RemoteException, UsuarioNaoLogadoException {
		try{
			this.servico.deslogar(sessao);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public IteratorBidirecional<Plano> getPlanos() throws RemoteException, PlanoInexistenteException{
		try{
			return this.servico.getPlanos();
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void cadastrarCliente(Cliente cliente, Telefone[] tarray) throws RemoteException, UsuarioExistenteException, EmailInvalidoException{
		try{
			this.servico.cadastrarCliente(cliente, tarray);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public Plano procurarPlano(String nome) throws RemoteException, PlanoInexistenteException {
		try{
			return this.servico.buscarPlano(nome);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public Plano procurarPlano(long id) throws PlanoInexistenteException, RemoteException{
		try{
			return this.servico.buscarPlano(id);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public IteratorBidirecional<ClassificacaoPreco> getClassificacoesPreco() throws RemoteException, ClassificacaoInexistenteException{
		try{
			return this.servico.getClassificacoes();
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public ClassificacaoPreco getClassificacao(String descricao) throws RemoteException, ClassificacaoInexistenteException {
		try{
			return this.servico.buscarClassificacao(descricao);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public ClassificacaoPreco getClassificacao(long id) throws RemoteException, ClassificacaoInexistenteException {
		try{
			return this.servico.buscarClassificacao(id);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void cadastrarProduto(long sessao, Produto p) throws RemoteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, ProdutoExistenteException, ProdutoInvalidoException {
		try{
			this.servico.cadastrarProduto(sessao, p);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void inserirLinkProduto(String nome, LinkFoto l) throws RemoteException, LinkFotoExistenteException, ProdutoInexistenteException{
		try{
			this.servico.inserirLinkFoto(nome, l);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public IteratorBidirecional<Produto> getProdutosRestricoes(CriterioBusca criterio) throws ProdutoInexistenteException, RemoteException{
		try{
			return this.servico.buscarProdutoRestricoes(criterio);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public IteratorBidirecional<Produto> getProdutosRecomendados() throws RemoteException, ProdutoInexistenteException{
		try{
			return this.servico.getProdutosRecomendados();
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void cadastrarPlano(long sessao, Plano plano) throws RemoteException, UsuarioSemPermissaoException, UsuarioNaoLogadoException, PlanoInvalidoException, PlanoExistenteException{
		try{
			this.servico.cadastrarPlano(sessao, plano);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	
	public Produto buscarProduto(String nomeProduto) throws ProdutoInexistenteException, RemoteException{
		try{
			return this.servico.buscarProduto(nomeProduto);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public Plano buscarPlano(String nome) throws PlanoInexistenteException, RemoteException{
		try{
			return this.servico.buscarPlano(nome);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public Plano buscarPlano(long id) throws PlanoInexistenteException, RemoteException{
		try{
			return this.servico.buscarPlano(id);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public Promocao buscarPromocao(String nome) throws PromocaoInexistenteException, RemoteException{
		try{
			return this.servico.buscarPromocao(nome);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public Promocao buscarPromocao(long id) throws PromocaoInexistenteException, RemoteException{
		try{
			return this.servico.buscarPromocao(id);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	
	public IteratorBidirecional<Promocao> getPromocoes() throws RemoteException, PromocaoInexistenteException{
		try{
			return this.servico.getPromocoes();
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}


	public void cadastrarClassificacao(Long sessao, ClassificacaoPreco c) throws RemoteException, UsuarioNaoLogadoException, ClassificacaoExistenteException, UsuarioSemPermissaoException {
		try{
			this.servico.inserirClassificacao(sessao, c);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void removerProduto(Long sessao, String codigoProduto) throws ProdutoInexistenteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, RemoteException {
		try{
			this.servico.descadastrarProduto(sessao, codigoProduto);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void removerUsuario(Long sessao, Long cpf, String senha) throws UsuarioInexistenteException, UsuarioSemPermissaoException, UsuarioNaoLogadoException, SenhaInvalidaException, RemoteException {
		try{
			this.servico.descadastrarUsuario(sessao, cpf, senha);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public IteratorBidirecional<AvaliacaoProduto> getAvaliacoes(Produto p) throws AvaliacaoInexistenteException, RemoteException{
		try{
			return this.servico.getAvaliacaoesProduto(p);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public boolean existeAvaliacao(long sessao, Produto p) throws RemoteException, UsuarioNaoLogadoException{
		try{
			return this.servico.buscarAvaliacaoProduto(sessao, p) != null;
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		} catch (AvaliacaoInexistenteException e) {
			return false;
		}
	}
	
	public AvaliacaoProduto getAvaliacao(long sessao, Produto p) throws RemoteException, UsuarioNaoLogadoException, AvaliacaoInexistenteException {
		try{
			return this.servico.buscarAvaliacaoProduto(sessao, p);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void cadastrarPromocao(Long sessao, Promocao promocao) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, PromocaoInvalidaException, PromocaoExistenteException, RemoteException {
		try{
			this.servico.cadastrarPromocao(sessao, promocao);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void removerPlano(Long l, String plano) throws PlanoInexistenteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, RemoteException {
		try{
			this.servico.descadastrarPlano(l, plano);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void removerPromocao(Long l, String nome) throws UsuarioSemPermissaoException, UsuarioNaoLogadoException, PromocaoInexistenteException, RemoteException {
		try{
			this.servico.descadastrarPromocao(l, nome);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void inserirAvaliacao(long sessao, AvaliacaoProduto a) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, AvaliacaoInvalidaException, RemoteException {
		try{
			this.servico.avaliarProduto(sessao, a);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		} catch (AvaliacaoExistenteException e) {
			try {
				this.servico.removerAvaliacaoProduto(sessao, a.getProduto());
				this.servico.avaliarProduto(sessao, a);
			} catch (AvaliacaoInexistenteException e1) {
				e1.printStackTrace();
			} catch (AvaliacaoExistenteException e1) {
				e.printStackTrace();
			}
		}
	}

	public void cadastrarUsuario(Long sessao, Usuario user) throws UsuarioSemPermissaoException, UsuarioExistenteException, UsuarioNaoLogadoException, RemoteException {
		try{
			this.servico.cadastrarUsuario(sessao, user);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public Filial getFilial(String id) throws FilialInexistenteException, RemoteException {
		try{
			return this.servico.buscarFilial(id);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public Filial getFilial(long id) throws FilialInexistenteException, RemoteException{
		try{
			return this.servico.buscarFilial(id);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public IteratorBidirecional<Filial> getFiliais() throws FilialInexistenteException, RemoteException{
		try{
			return this.servico.getFiliais();
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public IteratorBidirecional<Filial> getFiliais(long sessao, long cpf, long id) throws RemoteException, FilialInexistenteException, ProdutoInexistenteException, UsuarioNaoLogadoException, UsuarioInexistenteException, UsuarioSemPermissaoException, PeriodoInvalidoException{
		try{
			Calendar c = Calendar.getInstance(), c1;
			c1 = Calendar.getInstance();
			c1.set(c.get(Calendar.YEAR), c.get(Calendar.MONTH), c.get(Calendar.DAY_OF_MONTH), 23, 59);
			return this.servico.getFiliaisDisponiveis(this.servico.buscarUsuario(sessao, cpf), this.servico.buscarProduto(id), new Periodo(c,c1));
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}catch(Exception e){
			e.printStackTrace();
			return null;
		}
	}
	
	public IteratorBidirecional<Produto> getProdutos() throws ProdutoInexistenteException, RemoteException{
		try{
			return this.servico.getProdutos();
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void cadastrarCopia(long sessao, Copia copia) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, CopiaExistenteException, ProdutoInexistenteException, RemoteException {
		try{
			this.servico.cadastrarCopia(sessao, copia);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void removerCopia(Long sessao, int codigoCopia) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, ProdutoInexistenteException, CopiaInexistenteException, RemoteException {
		try{
			Copia c = this.servico.buscarCopia(codigoCopia);
			this.servico.descadastrarCopia(sessao, c.getProduto().getNome(), codigoCopia);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void alterarDadosUsuario(Long sessao, Usuario user) throws UsuarioNaoLogadoException, UsuarioInexistenteException, UsuarioSemPermissaoException, RemoteException, UsuarioExistenteException {
		try{
			this.servico.alterarDadosUsuario(sessao, user);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void cadastrarTelefonesCliente(long sessao, Usuario novo, Telefone[] tarray) throws UsuarioNaoLogadoException, UsuarioInexistenteException, UsuarioSemPermissaoException, RemoteException {
		try{
			this.servico.alterarTelefonesCliente(sessao, novo.getCpf(), tarray);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void cadastrarFilial(long sessao, Filial filial) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, FilialExistenteException, FilialInvalidaException, RemoteException {
		try{
			this.servico.cadastrarFilial(sessao, filial);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	
	public IteratorBidirecional<Reserva> buscarReservas(long sessao, Cliente cliente) throws RemoteException, ReservaInexistenteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException {
		try{
			return this.servico.buscarReservas(sessao, cliente);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	

	public void removerFilial(long sessao, String nome) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, FilialInexistenteException, RemoteException {
		try{
			this.servico.descadastrarFilial(sessao, nome);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void alterarTelefonesCliente(long sessao, Usuario novo, Telefone[] tarray) throws UsuarioNaoLogadoException, UsuarioInexistenteException, UsuarioSemPermissaoException, RemoteException {
		try{
			this.servico.alterarTelefonesCliente(sessao, novo.getCpf(), tarray);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void alterarProduto(Long sessao, Produto p) throws UsuarioSemPermissaoException, UsuarioNaoLogadoException, ProdutoInexistenteException, ProdutoInvalidoException, RemoteException, ProdutoExistenteException {
		try{
			this.servico.alterarDadosProduto(sessao, p);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void alterarPlano(Long sessao, Plano plano) throws UsuarioSemPermissaoException, UsuarioNaoLogadoException, PlanoInexistenteException, PlanoInvalidoException, RemoteException {
		try{
			this.servico.atualizarPlano(sessao, plano);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public Plano getPlano(long id) throws PlanoInexistenteException, RemoteException{
		try{
			return this.servico.buscarPlano(id);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public Produto buscarProduto(long id) throws ProdutoInexistenteException, RemoteException{
		try{
			return this.servico.buscarProduto(id);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}
	
	public Endereco getEndereco(long sessao, long cpf) throws UsuarioNaoLogadoException, UsuarioInexistenteException, UsuarioSemPermissaoException, RemoteException{
		try{
			return ((Cliente)this.getUsuario(sessao, cpf)).getEndereco();
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void alterarFilial(Long attribute, Filial filial) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, FilialInexistenteException, FilialInvalidaException, RemoteException {
		try{
			this.servico.atualizarFilial(attribute, filial);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public Copia getCopia(long id, long produto) throws CopiaInexistenteException, ProdutoInexistenteException, FilialInexistenteException, RemoteException {
		try{
			return this.servico.getCopiaFilial(this.buscarProduto(produto).getNome(), this.getFilial(id).getNome());
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void reservarCopia(long sessao, long cpf, Copia copia) throws UsuarioNaoLogadoException, ReservaExistenteException, UsuarioSemPermissaoException, CopiaLocadaNoPeriodoException, CopiaReservadaNoPeriodoException, ReservaInvalidaException, UsuarioInexistenteException, PeriodoInvalidoException, RemoteException, FilialNaoLogadaException, FilialInexistenteException {
		try{
			Calendar c = Calendar.getInstance();
			Calendar c1 = Calendar.getInstance();
			c1.add(Calendar.MINUTE, 30);
			
			this.servico.inserirReserva(sessao, new Reserva((Cliente) this.servico.buscarUsuario(sessao, cpf), copia, new Periodo(c, c1)));
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void inserirLocacao(long sessao, long cpf, Locacao loc) throws UsuarioNaoLogadoException, LocacaoExistenteException, LocacaoInvalidaException, UsuarioSemPermissaoException, CopiaLocadaNoPeriodoException, CopiaReservadaNoPeriodoException, RemoteException, FilialNaoLogadaException, UsuarioInexistenteException {
		try{
			this.servico.inserirLocacaoDomicilio(sessao, loc, ((Cliente) this.servico.buscarUsuario(sessao, cpf)).getEndereco());
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

	public void removerReserva(long sessao, long cpf, int codigoCopia) throws ReservaInexistenteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, UsuarioInexistenteException, CopiaInexistenteException, ProdutoInexistenteException, RemoteException {
		try{
			this.servico.removerReserva(sessao, cpf, codigoCopia);
		}catch(RemoteException e){
			Fachada.fachada = null;
			throw e;
		}
	}

}
