package servlets;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import negocio.exception.FilialInexistenteException;
import negocio.exception.PlanoInexistenteException;
import negocio.exception.SenhaInvalidaException;
import negocio.exception.UsuarioExistenteException;
import negocio.exception.UsuarioInexistenteException;
import negocio.exception.UsuarioNaoLogadoException;
import negocio.exception.UsuarioSemPermissaoException;
import util.Util;
import dados.exception.CpfInvalidoException;
import dados.exception.EmailInvalidoException;
import dados.exception.EnderecoInvalidoException;
import dados.exception.FilialInvalidaException;
import dados.exception.NomeInvalidoException;
import dados.exception.PlanoInvalidoException;
import dados.exception.TelefoneInvalidoException;
import dados.filial.Endereco;
import dados.usuario.Administrador;
import dados.usuario.Cliente;
import dados.usuario.Dependente;
import dados.usuario.Funcionario;
import dados.usuario.Plano;
import dados.usuario.Telefone;
import dados.usuario.Usuario;
import exception.ServidorIndisponivelException;
import fachada.Fachada;

/**
 * Servlet implementation class AlteracaoUsuario
 */
public class AlteracaoUsuario extends HttpServlet {
	private static final long serialVersionUID = 1L;

	public AlteracaoUsuario() {
		super();
	}

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		if(request.getSession().getAttribute("sessao") != null){
			try {
				long sessao = (Long) request.getSession().getAttribute("sessao");
				Fachada fachada = Fachada.getFachada();
				Usuario user = fachada.getUsuario(sessao, (Long) request.getSession().getAttribute("login"));

				if(user.getSenha().equals(request.getParameter("password"))){
					Usuario novo = fachada.getUsuario(sessao, Long.parseLong(request.getParameter("cpf").replace("-", "")));
					
					if(request.getParameter("tipo").equals("cliente")){
						Plano plano = fachada.procurarPlano(Long.parseLong(request.getParameter("plano")));

						if(request.getParameter("subtipo") != null){
							((Cliente)novo).setPlano(plano);						
						} else {
							novo = new Cliente(novo.getCpf(), novo.getSenha(), request.getParameter("nome"), request.getParameter("email"), plano, new Endereco(Util.htmlToString(request.getParameter("logradouro")), Util.htmlToString(request.getParameter("estado")), Util.htmlToString(request.getParameter("cidade")), Util.htmlToString(request.getParameter("bairro")), Util.htmlToString(request.getParameter("complemento")), Integer.parseInt(request.getParameter("numero")), Integer.parseInt(request.getParameter("cep").replace("-", ""))));
							
							Telefone t = new Telefone(Integer.parseInt(request.getParameter("dddtelefone")), Integer.parseInt(request.getParameter("telefone").replace("-", ""))), t2 = null;
							
							if(request.getParameter("dddtelefone2") != null && !request.getParameter("dddtelefone2").isEmpty() && request.getParameter("telefone2") != null && !request.getParameter("telefone2").isEmpty()){
								t2 = new Telefone(Integer.parseInt(request.getParameter("dddtelefone2")), Integer.parseInt(request.getParameter("telefone2").replace("-", "")));
							}
							
							Telefone[] tarray = new Telefone[t2 == null ? 1 : 2];
							tarray[0] = t;
							
							if(t2 != null){
								tarray[1] = t2;
							}
							
							((Cliente)novo).setTelefones(tarray);
						}
					} else if(request.getParameter("tipo").equals("dependente")){
						novo = new Dependente(novo.getCpf(),novo.getSenha(), Util.htmlToString(request.getParameter("nome")), Util.htmlToString(request.getParameter("email")), (Cliente) fachada.getUsuario(sessao, Long.parseLong(Util.htmlToString(request.getParameter("cpf-cliente")))));
					} else if(request.getParameter("tipo").equals("dependente")){
						novo = new Funcionario(novo.getCpf(),novo.getSenha(), Util.htmlToString(request.getParameter("nome")), Util.htmlToString(request.getParameter("email")), fachada.getFilial(Util.htmlToString(request.getParameter("filial"))));
					} else if(request.getParameter("tipo").equals("administrador")){
						novo = new Administrador(novo.getCpf(),novo.getSenha(), Util.htmlToString(request.getParameter("nome")), Util.htmlToString(request.getParameter("email")));
					} else {
						response.sendRedirect("cadastro-erro.jsp?tipo=usuario&erro=campo");
					}

					Fachada.getFachada().alterarDadosUsuario(sessao, novo);
					response.sendRedirect("cadastro-concluido.jsp?tipo=alteracao");
				} else {
					response.sendRedirect("cadastro-erro.jsp?erro=senhaIncorreta");
				}
			} catch (NumberFormatException e) {
				response.sendRedirect("cadastro-erro.jsp?erro=campo");
			} catch (EmailInvalidoException e) {
				response.sendRedirect("cadastro-erro.jsp?erro=email");
			} catch (EnderecoInvalidoException e) {
				response.sendRedirect("cadastro-erro.jsp?erro=endereco");
			} catch (PlanoInexistenteException e) {
				response.sendRedirect("cadastro-erro.jsp?erro=plano");
			} catch (ServidorIndisponivelException e) {
				response.sendRedirect("erro-servidor.jsp");
			} catch (UsuarioExistenteException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=usuario&erro=replicaEmail");
			} catch (TelefoneInvalidoException e) {
				response.sendRedirect("cadastro-erro.jsp?erro=telefone");
			} catch (NomeInvalidoException e) {
				response.sendRedirect("cadastro-erro.jsp?erro=nome");
			} catch (SenhaInvalidaException e) {
				response.sendRedirect("cadastro-erro.jsp?erro=senha");
			} catch (CpfInvalidoException e) {
				e.printStackTrace();
				response.sendRedirect("index.jsp");
			} catch (PlanoInvalidoException e) {
				response.sendRedirect("erro-servidor.jsp");
			} catch (UsuarioNaoLogadoException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=produto&erro=usuarioNaoLogado");
			} catch (UsuarioInexistenteException e) {
				e.printStackTrace();
				response.sendRedirect("index.jsp");
			} catch (UsuarioSemPermissaoException e) {
				e.printStackTrace();
				response.sendRedirect("index.jsp");
			} catch (FilialInvalidaException e) {
				e.printStackTrace();
				response.sendRedirect("index.jsp");
			} catch (FilialInexistenteException e) {
				e.printStackTrace();
				response.sendRedirect("index.jsp");
			}
		}else{
			response.sendRedirect("index.jsp");
		}
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		this.doGet(request, response);
	}

}
