package servlets;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import negocio.exception.PlanoExistenteException;
import negocio.exception.UsuarioNaoLogadoException;
import negocio.exception.UsuarioSemPermissaoException;

import util.Util;

import dados.exception.LinkFotoInvalidoException;
import dados.exception.PlanoInvalidoException;
import dados.produto.LinkFoto;
import dados.produto.TipoLinkFoto;
import dados.usuario.Plano;
import exception.ServidorIndisponivelException;
import fachada.Fachada;

public class CadastroPlano extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
    public CadastroPlano() {
        super();
    }

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		Object o = request.getSession().getAttribute("sessao");
		
		if(o != null){
			Long sessao = (Long) o;
			
			try {
				Plano plano = new Plano(Double.parseDouble(Util.htmlToString(request.getParameter("mensalidade"))), Util.htmlToString(request.getParameter("nome")), Util.htmlToString(request.getParameter("descricao")), Integer.parseInt(Util.htmlToString(request.getParameter("locacao"))), new LinkFoto(TipoLinkFoto.FOTOBOX,Util.htmlToString(request.getParameter("link"))));
				Fachada.getFachada().cadastrarPlano(sessao, plano);
				
				response.sendRedirect("cadastro-concluido.jsp?tipo=plano");
			} catch (NumberFormatException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=plano&erro=campo");
			} catch (PlanoInvalidoException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=plano&erro=planoInvalido");
			} catch (LinkFotoInvalidoException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=plano&erro=linkFoto");
			} catch (UsuarioSemPermissaoException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=plano&erro=permissaoNegada");
			} catch (UsuarioNaoLogadoException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=produto&erro=usuarioNaoLogado");
			} catch (PlanoExistenteException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=plano&erro=planoExistente");
			} catch (ServidorIndisponivelException e) {
				response.sendRedirect("erro-servidor.jsp");
			} catch (NullPointerException e){
				e.printStackTrace();
				response.sendRedirect("index.jsp");
			}
		}else{
			response.sendRedirect("index.jsp");
		}
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		this.doGet(request, response);
	}

}
