package servlets;

import java.io.IOException;
import java.util.List;
import java.util.Vector;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import util.Util;

import negocio.exception.ClassificacaoInexistenteException;
import negocio.exception.ProdutoExistenteException;
import negocio.exception.UsuarioNaoLogadoException;
import negocio.exception.UsuarioSemPermissaoException;

import dados.exception.LinkFotoInvalidoException;
import dados.exception.ProdutoInvalidoException;
import dados.produto.Documentario;
import dados.produto.Filme;
import dados.produto.Jogo;
import dados.produto.LinkFoto;
import dados.produto.Musica;
import dados.produto.Produto;
import dados.produto.Serie;
import dados.produto.TipoLinkFoto;
import exception.ServidorIndisponivelException;
import fachada.Fachada;

public class CadastroProduto extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
    public CadastroProduto() {
        super();
    }

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try{
			String tipo = Util.htmlToString(request.getParameter("type"));
			String nome = Util.htmlToString(request.getParameter("nome"));
			
			String genero = Util.htmlToString(request.getParameter("genero"));

			String censura = Util.htmlToString(request.getParameter("censura"));
			String descricao = Util.htmlToString(request.getParameter("descricao"));
			String empresa = Util.htmlToString(request.getParameter("empresa"));
			
			int ano = Integer.parseInt(request.getParameter("ano"));
			String trailer = Util.htmlToString(request.getParameter("trailer"));
			String poster = Util.htmlToString(request.getParameter("poster"));
			String box = Util.htmlToString(request.getParameter("box"));
			List<LinkFoto> lista = new Vector<LinkFoto>();
			
			if(poster != null && !poster.equals("")){
				lista.add(new LinkFoto(TipoLinkFoto.POSTER, box));
			}else{
				response.sendRedirect("cadastro-erro.jsp?tipo=produto&erro=poster");
				return;
			}
			
			if(box != null && !box.trim().equals("")){
				lista.add(new LinkFoto(TipoLinkFoto.FOTOBOX, box));
			}else{
				response.sendRedirect("cadastro-erro.jsp?tipo=produto&erro=linkFoto");
				return;
			}
			
			for(int i = 1; i <= 4; i++){
				if(request.getParameter("imagem"+i) != null && !request.getParameter("imagem"+i).isEmpty()){
					lista.add(new LinkFoto(TipoLinkFoto.NORMAL, request.getParameter("imagem"+i)));
				}
			}
			
			Produto p = null;
			
			if(tipo.equals("filme")){
				p = new Filme(nome,genero, censura, descricao, empresa, trailer, Util.htmlToString(request.getParameter("diretor")), Util.htmlToString(request.getParameter("autor")), Fachada.getFachada().getClassificacao(request.getParameter("classificacao")), ano, Integer.parseInt(request.getParameter("duracao")));
			}else if(tipo.equals("serie")){
				p = new Serie(nome,genero, censura, descricao, empresa, trailer, Fachada.getFachada().getClassificacao(request.getParameter("classificacao")), Integer.parseInt(request.getParameter("ano")), Integer.parseInt(request.getParameter("temporada")), Integer.parseInt(request.getParameter("inicio")), Integer.parseInt(request.getParameter("final")));
			}else if(tipo.equals("musica")){
				p = new Musica(nome,genero, censura, Util.htmlToString(request.getParameter("artista")), descricao, empresa, trailer, Fachada.getFachada().getClassificacao(request.getParameter("classificacao")), Integer.parseInt(request.getParameter("ano")));
			}else if(tipo.equals("jogo")){
				p = new Jogo(nome,genero, censura, descricao, Util.htmlToString(request.getParameter("console")), empresa, trailer, Fachada.getFachada().getClassificacao(request.getParameter("classificacao")), Integer.parseInt(request.getParameter("ano")), Integer.parseInt(request.getParameter("jogadores")));
			}else if(tipo.equals("documentario")){
				p = new Documentario(nome,genero, censura, descricao, Util.htmlToString(request.getParameter("tema")), empresa, trailer, Fachada.getFachada().getClassificacao(request.getParameter("classificacao")), Integer.parseInt(request.getParameter("ano")));
			}else{
				response.sendRedirect("index.jsp");
				return;
			}
			
			Fachada.getFachada().cadastrarProduto((Long)request.getSession().getAttribute("sessao"),p);

			for(LinkFoto l : lista){
				Fachada.getFachada().inserirLinkProduto(p.getNome(), l);
			}
			
			response.sendRedirect("cadastro-concluido.jsp?tipo=produto");
		}catch(NullPointerException e){
			response.sendRedirect("index.jsp");
		} catch (ProdutoInvalidoException e) {
			response.sendRedirect("cadastro-erro.jsp?tipo=produto&erro=produtoInvalido");
		} catch (ClassificacaoInexistenteException e) {
			response.sendRedirect("cadastro-erro.jsp?tipo=produto&erro=classificacaoInexistente");
		} catch (ServidorIndisponivelException e) {
			response.sendRedirect("erro-servidor.jsp");
		} catch (LinkFotoInvalidoException e) {
			response.sendRedirect("cadastro-erro.jsp?tipo=produto&erro=linkFoto");
		} catch (UsuarioNaoLogadoException e) {
			response.sendRedirect("cadastro-erro.jsp?tipo=produto&erro=usuarioNaoLogado");
		} catch (UsuarioSemPermissaoException e) {
			response.sendRedirect("cadastro-erro.jsp?tipo=produto&erro=permissaoNegada");
		} catch (ProdutoExistenteException e) {
			response.sendRedirect("cadastro-erro.jsp?tipo=produto&erro=produtoExistente");
		} catch(Exception e){
			e.printStackTrace();
			response.sendRedirect("cadastro-erro.jsp?tipo=produto&erro=campo");
		}
	}

}
