package servlets;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import negocio.exception.PromocaoExistenteException;
import negocio.exception.UsuarioNaoLogadoException;
import negocio.exception.UsuarioSemPermissaoException;
import util.Util;
import dados.exception.LinkFotoInvalidoException;
import dados.exception.PromocaoInvalidaException;
import dados.locacao.Promocao;
import dados.produto.LinkFoto;
import dados.produto.TipoLinkFoto;
import exception.ServidorIndisponivelException;

import fachada.Fachada;

/**
 * Servlet implementation class CadastroPromocao
 */
public class CadastroPromocao extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
    public CadastroPromocao() {
        super();
    }

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		Object o = request.getSession().getAttribute("sessao");
		
		if(o != null){
			Long sessao = (Long) o;
			
			try {
				Promocao promocao = new Promocao(Util.htmlToString(request.getParameter("nome")), Util.htmlToString(request.getParameter("descricao")), new LinkFoto(TipoLinkFoto.FOTOBOX,Util.htmlToString(request.getParameter("link"))));
				Fachada.getFachada().cadastrarPromocao(sessao, promocao);
				
				response.sendRedirect("cadastro-concluido.jsp?tipo=promocao");
			} catch (NumberFormatException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=promocao&erro=campo");
			} catch (LinkFotoInvalidoException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=promocao&erro=linkFoto");
			} catch (UsuarioSemPermissaoException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=promocao&erro=permissaoNegada");
			} catch (UsuarioNaoLogadoException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=produto&erro=usuarioNaoLogado");
			} catch (ServidorIndisponivelException e) {
				response.sendRedirect("erro-servidor.jsp");
			} catch (NullPointerException e){
				e.printStackTrace();
				response.sendRedirect("index.jsp");
			} catch (PromocaoInvalidaException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=promocao&erro=promocaoInvalida");
			} catch (PromocaoExistenteException e) {
				response.sendRedirect("cadastro-erro.jsp?tipo=promocao&erro=promocaoExistente");
			}
		}else{
			response.sendRedirect("index.jsp");
		}
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		this.doGet(request, response);
	}

}
